<?php
/**
 *
 * @package phpBB Extension - Acme Demo
 * @copyright (c) 2013 phpBB Group
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 *
 */

namespace topicit\topicit\event;

/**
 * @ignore
 */
use Emojione\Emojione;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event listener
 */
class main_listener implements EventSubscriberInterface
{
    /* @var \phpbb\controller\helper */
    protected $helper;

    /* @var \phpbb\template\template */
    protected $template;

    /* @var \phpbb\user */
    protected $user;

    /* @var \phpbb\auth\auth */
    protected $auth;

    /**
     * Constructor
     *
     * @param \phpbb\controller\helper   $helper     Controller helper object
     * @param \phpbb\template            $template   Template object
     */
    public function __construct(\phpbb\template\template $template, \phpbb\user $user, \phpbb\auth\auth $auth)
    {
        $this->template = $template;
        $this->user     = $user;
        $this->auth     = $auth;
    }

    public static function getSubscribedEvents()
    {
        return array(
            'core.user_setup'           => 'setup',
            'core.page_header_after'    => 'header',
            'core.page_footer_after'    => 'setupConnect',
            'core.submit_post_end'      => 'submitPostEnd', // NEW TOPIC & POST 
            'core.approve_posts_after'  => 'approvePostAfter', // NEW POST (AFTER APPROVE)
            'core.approve_topics_after' => 'approveTopicsAfter', // NEW TOPIC (AFTER APPROVE)
            'core.submit_pm_after'      => 'submitPmAfter', // NEW MP
            'core.ucp_add_zebra'        => 'ucpAddZebra', // NEW FRIEND
            'core.delete_user_after'    => 'delete', // USER DELETION
            'core.viewtopic_modify_post_row'       => 'convertEmoji',
            'core.login_box_before'       => 'initTopicitCnnect',

        );
    }

    public function initTopicitCnnect($event)
    {
        define('SHOW_LOGIN_FORM_TI_BUTTON', true);
    }

    public function convertEmoji($event)
    {
        global $row, $config, $rowset_data,$phpbb_root_path,$post_row;

        if(defined('IN_TOPICIT'))
        {
            return ;
        }

        include_once $phpbb_root_path . '/topicit/classes/emojione/autoload.php';

        $data = $event->get_data();

        $message = Emojione::getClient()->shortnameToImage($data['post_row']['MESSAGE']);

        if(!empty($message))
        {
            $data['post_row']['MESSAGE'] = $post_row['MESSAGE'] = $message;
        }
        $event->set_data($data);
    }

    public function setupConnect($event)
    {
        global $config,$phpbb_root_path;

        $errors = base64_decode(request_var('topicit_errors', ''));
        if (!empty($errors)) {
            $this->template->assign_var ('TOPICIT_LOGIN_ERRORS', $errors);
        }

        // APP
        if (!class_exists('App')) {
            require_once $phpbb_root_path . 'topicit/App.php';
        }
        $app = \App::getInstance();

        // ACTIVE FOR ANONS ONLY
        if (empty ($this->user->data['user_id']) || $this->user->data['user_id'] == ANONYMOUS) {
            if ($config['topicit_login']) {

                if(defined('SHOW_LOGIN_FORM_TI_BUTTON')) {
                    $this->template->assign_var ('TOPICIT_SHOW_BUTTON', true);
                }

                $this->template->assign_var ('TOPICIT_LOGIN_EMBED_SOCIAL_LOGIN', 1);
                $this->template->assign_var ('TOPICIT_URL', $app->config('homeURL') . '/');
                $this->template->assign_var ('TOPICIT_LOGIN_CALLBACK_URI', self::get_current_url ());
                $this->template->assign_var ('TOPICIT_LOGIN_TITLE', $this->user->lang['PLUG_TOPICIT_CONNECT_BUTTON']);
                $this->template->assign_var ('USER_LANGUAGE', strtolower(substr($this->user->lang['USER_LANG'], 0, 2)));
                $this->template->assign_var ('TOPICIT_LOGIN_CAPTION', $this->user->lang['PLUG_TOPICIT_CONNECT_EXPLAIN']);
            }
        }
    }

    private static function is_https_on()
    {
        global $request;

        if ($request->server ('SERVER_PORT') == 443) {
            return true;
        }

        if ($request->server ('HTTP_X_FORWARDED_PROTO') == 'https') {
            return true;
        }

        if (in_array(strtolower(trim($request->server ('HTTPS'))), array ('on', '1'))) {
            return true;
        }

        return false;
    }

    private static function get_current_url($remove_vars = array ('sid'))
    {
        global $request;

        // Extract Uri
        if (strlen(trim($request->server('REQUEST_URI'))) > 0) {
            $request_uri = trim($request->server('REQUEST_URI'));
        } else {
            $request_uri = trim($request->server('PHP_SELF'));
        }

        $request_uri = htmlspecialchars_decode($request_uri);

        // Extract Protocol
        if (self::is_https_on ()) {
            $request_protocol = 'https';
        } else {
            $request_protocol = 'http';
        }

        // Extract Host
        if (strlen(trim($request->server('HTTP_X_FORWARDED_HOST'))) > 0) {
            $request_host = trim($request->server('HTTP_X_FORWARDED_HOST'));
        } elseif (strlen(trim ($request->server('HTTP_HOST'))) > 0) {
            $request_host = trim($request->server('HTTP_HOST'));
        } else {
            $request_host = trim($request->server('SERVER_NAME'));
        }

        // Port of this request
        $request_port = '';

        // We are using a proxy
        if (strlen(trim ($request->server('HTTP_X_FORWARDED_PORT'))) > 0) {
            $request_port = intval ($request->server('HTTP_X_FORWARDED_PORT'));
        } elseif (strlen(trim ($request->server('SERVER_PORT'))) > 0) {
            $request_port = intval ($request->server('SERVER_PORT'));
        }

        // Remove standard ports
        $request_port = (!in_array($request_port, array(80, 443)) ? $request_port : '');

        // Build url
        $current_url = $request_protocol . '://' . $request_host . (!empty($request_port) ? (':' . $request_port) : '') . $request_uri;

        // Remove query arguments.
        if (is_array($remove_vars) && count($remove_vars) > 0) {
            // Break up url
            list($url_part, $query_part) = array_pad(explode('?', $current_url), 2, '');
            parse_str($query_part, $query_vars);

            // Remove argument.
            if (is_array($query_vars)) {
                foreach ($remove_vars as $var) {
                    if (isset($query_vars [$var])) {
                        unset($query_vars [$var]);
                    }
                }

                // Build new url
                $current_url = $url_part . ((is_array($query_vars) && count($query_vars) > 0) ? ('?' . http_build_query($query_vars)) : '');
            }
        }

        // Done
        return $current_url;
    }

    public function setup($event)
    {
        $lang_set_ext = $event['lang_set_ext'];
        $lang_set_ext[] = array(
            'ext_name' => 'topicit/topicit',
            'lang_set' => 'common',
        );
        $event['lang_set_ext'] = $lang_set_ext;
    }

    public function approvePostAfter($event)
    {
        global $config,$phpbb_root_path;

        if (!empty($config['topicit_key'])) {

            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
                \App::getInstance();
            }

            $data = $event->get_data();

            if ($data['action'] == 'approve') {
                foreach ($data['post_info'] as $post) {
                    \PushController::newReply($post, true);
                }
            }
        }
    }

    public function approveTopicsAfter($event)
    {
        global $config,$phpbb_root_path;

        if (!empty($config['topicit_key'])) {
            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
                \App::getInstance();
            }

            $data = $event->get_data();

            if ($data['action'] == 'approve') {
                foreach ($data['topic_info'] as $post) {
                    \PushController::newTopic($post, true);
                }
            }
        }
    }

    public function submitPostEnd($event)
    {
        global $config,$phpbb_root_path;

        if (!empty($config['topicit_key'])) {
            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
                \App::getInstance();
            }

            $data = $event->get_data();

            switch ($data['mode']) {
                case 'post':
                    \PushController::newTopic($data);
                    break;

                case 'reply':
                case 'quote':
                    \PushController::newReply($data);
                    break;
            }
        }
    }

    public function submitPmAfter($event)
    {
        global $config,$phpbb_root_path;

        if (!empty($config['topicit_key'])) {
            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
                \App::getInstance();
            }

            \PushController::newPm($event->get_data());
        }
    }

    public function ucpAddZebra($event)
    {
        global $config,$phpbb_root_path;

        if (!empty($config['topicit_key'])) {
            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
                \App::getInstance();
            }

            \PushController::newFriend($event->get_data());
        }
    }

    public function header($event)
    {
        global $config, $topic_data, $user, $phpbb_root_path;

        // PARAMS
        $appIdIos     = '1272582179';
        $appIdAndroid = 'net.topicit.android';
        $appName      = "Topic'it";
        $twitterCard  = $user->lang['PLUG_TOPICIT_TWITTERCARD'];

        // LOCATION
        $appLocation = $this->getLocation();

        // HEADER
        $header = '';
        if (in_array($appLocation['location'], array('forum', 'topic', 'post', 'home', 'profile')) && $appLocation['host']) {
            // APP INDEXING
            $header .= "<link href=\"android-app://{$appIdAndroid}/topicit/{$appLocation['host']}\" rel=\"alternate\" />
            <link href=\"ios-app://{$appIdIos}/topicit/{$appLocation['host']}\" rel=\"alternate\" />\n";

            // DEEP LINKING
            if ($config['topicit_deeplinking_enabled']) {

                $eventData = $event->get_data();

                // PAGE TITLE - DESCRIPTION
                $pageTitle = (in_array($appLocation['location'], array('topic')) ? $this->htmlEscape($topic_data['topic_title'], true) . ' - ' . $config['sitename'] : (in_array($appLocation['location'], array('forum')) ? $this->htmlEscape($eventData['page_title'], true) . ' - ' . $config['sitename'] : $config['sitename'] . ' - ' . $this->htmlEscape($eventData['page_title'], true)));
                $pageDescription = $this->htmlEscape($config['site_desc'], true);

                // APP LINKS
                $header .= "<meta property=\"al:android:package\" content=\"{$appIdAndroid}\" />
                <meta property=\"al:android:url\" content=\"{$appLocation['url']}\" />
                <meta property=\"al:android:app_name\" content=\"{$appName}\" />
                <meta property=\"al:ios:url\" content=\"{$appLocation['url']}\" />
                <meta property=\"al:ios:app_store_id\" content=\"{$appIdIos}\" />
                <meta property=\"al:ios:app_name\" content=\"{$appName}\" />\n";

                // TWITTER CARD
                $header .= "<meta name=\"twitter:card\" content=\"app\" />
                <meta name=\"twitter:site\" content=\"" . (!empty($config['topicit_twitter_site']) ? $config['topicit_twitter_site'] : '@' . $twitterCard) . "\" />
                <meta name=\"twitter:title\" content=\"{$pageTitle}\" />
                <meta name=\"twitter:description\" content=\"{$pageDescription}\" />
                <meta name=\"twitter:app:id:iphone\" content=\"{$appIdIos}\" />
                <meta name=\"twitter:app:url:iphone\" content=\"{$appLocation['url']}\" />
                <meta name=\"twitter:app:id:ipad\" content=\"{$appIdIos}\" />
                <meta name=\"twitter:app:url:ipad\" content=\"{$appLocation['url']}\" />
                <meta name=\"twitter:app:id:googleplay\" content=\"{$appIdAndroid}\" />
                <meta name=\"twitter:app:url:googleplay\" content=\"{$appLocation['url']}\" />\n";
            }
        }

        // SMART APP BANNER //
        if ($config['topicit_app_banner_enable']) {

            // APP
            if (!class_exists('App')) {
                require_once $phpbb_root_path . 'topicit/App.php';
            }
            $app = \App::getInstance();

            $params = array(
                'url'   => base64_encode(generate_board_url()),
                'email' => base64_encode($user->data['user_email']),
                'lang'  => base64_encode($user->lang['USER_LANG'])
            );
            $installIos = $app->config('homeURL') . '/sab?' . http_build_query(array_merge($params, array('os' => base64_encode('ios'))));
            $installAndroid = $app->config('homeURL') . '/sab?' . http_build_query(array_merge($params, array('os' => base64_encode('android'))));

            $base = basename(dirname(dirname(__FILE__)));

            $header .= "<meta name=\"apple-itunes-app\" content=\"app-id={$appIdIos}\">
            <meta name=\"google-play-app\" content=\"app-id={$appIdAndroid}\">
            <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">
        
            <link rel=\"stylesheet\" href=\"./{$base}/smart-app-banner/smart-app-banner.css\" type=\"text/css\" media=\"screen\" />
            <link rel=\"apple-touch-icon\" href=\"./{$base}/smart-app-banner/apple-touch-icon.png\" />
            <link rel=\"android-touch-icon\" href=\"./{$base}/smart-app-banner/android-icon.png\" />
            <script src=\"./{$base}/smart-app-banner/smart-app-banner.js\"></script>
            <script type=\"text/javascript\">
            var userAgent = navigator.userAgent, overrideType;
            overrideType = '';
            if(/CriOS/i.test(navigator.userAgent) && /iphone|ipod|ipad/i.test(navigator.userAgent)){
                overrideType = 'ios';
            }
            if (userAgent.match(/Android/i) !== null) {
                overrideType = 'android';
            }

            if(overrideType){
              new SmartBanner({
                  daysHidden: 15,
                  daysReminder: 90,
                  appStoreLanguage: \"{$user->lang['USER_LANG']}\",
                  title: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_TITLE']}\",
                  author: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_AUTHOR']}\",
                  button: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_BUTTON']}\",
                  store: {
                      ios: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_STORE_IOS']}\",
                      android: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_STORE_ANDROID']}\",
                  },
                  price: {
                      ios: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_PRICE_IOS']}\",
                      android: \"{$user->lang['PLUG_TOPICIT_SMARTBANNER_PRICE_ANDROID']}\",
                  }
                   , force: overrideType 
              });
              
                setTimeout(replaceBanner, 1000);
                
                function replaceBanner() { 
                    try {
                        var regex = /smartbanner\ssmartbanner-(android|ios)/; 
                        var banner = document.getElementsByClassName(\"smartbanner\") || [];

                        if(banner.length > 0) {
                            var str = banner[0].getAttribute(\"class\");
                            var bannerVer = regex.exec(str);
                            var url = ((bannerVer[1] == \"android\") ? \"{$installAndroid}\" : \"{$installIos}\" );

                            var bannerBtn = document.getElementsByClassName(\"smartbanner-button\") || [];

                            if(bannerBtn.length > 0) {
                                bannerBtn[0].setAttribute(\"href\", url);
                            }
                        }
                    } catch(e) {
                        if(console) {
                            console.error(e);
                        }
                    }
                }
            }
            </script>";
        }

        if (!empty($header)) {
            $this->template->append_var("META", $header);
        }
    }

    public function delete($event)
    {
        global $phpbb_root_path;

        // APP
        if (!class_exists('App')) {
            require_once $phpbb_root_path . 'topicit/App.php';
        }
        \App::getInstance();

        $eventData = $event->get_data();
        foreach ($eventData['user_ids'] as $uid) {
            \UserFunctions::unsetTopicitUser($uid);
        }
    }

    private function getLocation()
    {
        global $config, $user, $phpbb_root_path, $request;

        $request->enable_super_globals();

        $params   = array();
        $location = $user->extract_current_page($phpbb_root_path);

        $params['location'] = 'home';

        switch ($location['page_name']) {
            case "viewforum.php":
                $params['location'] = 'forum';

                if (!empty($_GET['f'])) {
                    $params['fid'] = $_GET['f'];
                    $params['page'] =  1;
                    $params['perpage'] = (int)$config['topics_per_page'];

                    if (isset($_GET['start'])) $params['page'] = ($_GET['start']/$config['topics_per_page']) + 1;
                }
                break;
            case "index.php":
            case '':
                $params['location'] = 'index';
                break;
            case "ucp.php":
                if (!empty($_GET['i']) && ($_GET['i'] == "pm")) {
                    $params['location'] = 'message';
                    if(!empty($_GET['p'])) $params['mid'] = $_GET['p'];
                }

                /*if (!empty($_GET['mode']) && ($_GET['mode'] == 'login')) {
                    $params['location'] = 'login';
                }*/
                break;
            /*case "search.php":
                $params['location'] = "search";
                break;*/
            case "viewtopic.php":
                if (!empty($_GET['t'])) {
                    $params['location'] = 'topic';
                    $params['tid']      = $_GET['t'];
                    $params['page']     =  1;
                    $params['perpage']  = (int)$config['posts_per_page'];

                    if (isset($_GET['f'])) $params['fid'] = $_GET['f'];
                    if (isset($_GET['start'])) $params['page'] = ($_GET['start'] / $config['posts_per_page']) + 1;

                    /*if (isset($_GET['p'])) {
                        $params['pid']      = $_GET['p'];
                        $params['location'] = 'post';
                    }*/
                }
                break;
            case "memberlist.php":
                if (!empty($_GET['mode']) && $_GET['mode'] == "viewprofile" && !empty($_GET['u'])) {
                    $params['location'] = 'profile';
                    $params['uid']      = $_GET['u'];
                }

                break;
            /*case "viewonline.php":
                $params['location'] = 'online';
                break;*/
        }

        // URL
        $url = generate_board_url() . '/?' . http_build_query($params);
        $url = preg_replace('/^(https|http)/isU', 'topicit', $url);
        $url = preg_match('#^topicit://#i', $url) ? $url : 'topicit://';

        // HOST
        $host = preg_replace('#topicit://#si', '', $url);

        // LOCATION
        if (strpos($location['query_string'], 'view=next') !== false || strpos($location['query_string'], 'view=previous') !== false || strpos($location['query_string'], 'view=print') !== false) {
            $params['location'] = "home";
        }

        return array(
            'location' => $params['location'],
            'host'     => $host,
            'url'      => $url
        );
    }

    private function htmlEscape($str)
    {
        return str_replace('&amp;', '&', addslashes(htmlspecialchars($str, ENT_NOQUOTES, "UTF-8")));
    }
}
